% Computes the windowed fft of the audiofile
% windowsize is the size of the FFT window in seconds
% overlap is the nubmer of windows that overlap, so an overlap of 32
% implies that the shift between windows is 1/32 of a window
% recommended values for sound detection: 
%  fs = 8000, minfreq = 50, maxfreq = 4000, numbands = 32,
%  windowsize = 1024, overlap = 16

function bands = compute_windowed_fft(wav, fs, freq_bounds, windowsize, overlap)

blocks = makeblocks(wav, windowsize, overlap);
numblocks = length(blocks);

minfreq = freq_bounds(1);
maxfreq = freq_bounds(end);

tl = length(block2fft(wav(blocks(1) : blocks(1) + windowsize - 1),  fs, minfreq, maxfreq));
p = zeros(tl, numblocks);

hann_window(1:windowsize, 1) = 0.54 + 0.46*cos(2*pi*(0:windowsize-1)/(windowsize-1));

for (i = 1:numblocks)
  p(:, i) = block2fft(wav(blocks(i) : blocks(i) + windowsize - 1).*hann_window, fs, minfreq, maxfreq);
end

%bands = power2band(p, numbands, minfreq, maxfreq);

%x = exp(log(maxfreq / minfreq) / (numbands));

seconds = size(p,1)/(maxfreq-minfreq);

bands = zeros(length(freq_bounds)-1, size(p, 2));

for (i = 1:length(freq_bounds)-1)

  %tstart = floor(((x^i)-1)*minfreq*seconds+1);
  %tend = floor((x^(i+1)-1)*minfreq*seconds);  
  tstart = floor((freq_bounds(i)-minfreq)*seconds+1);
  tend = floor((freq_bounds(i+1)-minfreq)*seconds);
  bands(i, :) = sum( p(tstart : tend, :), 1 );
  
end
