function [data, labels, confidences] = ...
        create_uniform_length_clips(pos_clips, neg_clips, num_synth, num_neg, med_length, varargin)

    num_prev_stages = 0;
    num_var_args = length(varargin) ;   
    if num_var_args > 1        
        num_prev_stages = varargin{1}(1);
        prev_classifiers = varargin{2}(:);
    end

    num_pos_orig = length(pos_clips);
    num_pos = num_pos_orig*max(num_synth, 1);
    num_neg_clips = length(neg_clips);

    labels = [ones(num_pos, 1) ; -ones(num_neg, 1)];
    
    data = zeros(med_length, num_pos+num_neg);
    
    if num_prev_stages > 0
        confidences = zeros(num_pos+num_neg, 1);
    else
        confidences = 0;
    end
    
    % create positive clips
    sample_num = 0;
    for p = 1:num_pos_orig
        for s = 1:max(num_synth, 1)                                
            
            if num_synth > 0 % use synthetic background                                    
                % between 5% and 25% noise
                background_mixing_coef = 0.20 * rand(1) + 0.05;
            else
                background_mixing_coef = 0;
            end
            sample_num = sample_num + 1;
            
            data(:, sample_num) = embed_clip(pos_clips(p).data, med_length, neg_clips, background_mixing_coef);
            
            if num_prev_stages > 0
                [confidences(sample_num), passed] = test_single_clip(data(:, sample_num), prev_classifiers);
                if ~passed
                    sample_num = sample_num - 1;
                end
            end
            
        end
    end
    if sample_num < num_pos
        data = data(:, 1:sample_num+num_neg);
        confidences = confidences(1:(sample_num+num_neg), 1);
        num_pos = sample_num;
    end
    
    % create negative clips
    if num_prev_stages == 0
        for n = 1:num_neg
            rc = ceil(rand(1)*num_neg_clips);
            i1 = ceil(rand(1)*(length(neg_clips(rc).data)-med_length));
            i2 = i1 + med_length - 1;
        
            sample_num = n + num_pos;
            data(:, sample_num) = neg_clips(rc).data(i1:i2);
        end        
    else 
        num_passed = 0;
        data_locations = zeros(0, 3);
        neg_confidences = [];
        for t = 1:length(neg_clips)            
            [passed_times, passed_data(t).data_locations, tmp_confidences] = test_track(neg_clips(t).data, prev_classifiers, '');            
            data_locations = [data_locations ; [t*ones(length(tmp_confidences), 1) passed_data(t).data_locations]];
            neg_confidences = [neg_confidences ; tmp_confidences];
            num_passed = num_passed + length(tmp_confidences);
            if (num_passed > 2*num_neg)
                break;
            end            
        end        
        if num_passed > num_neg
            data_locations = data_locations(floor(1:num_passed/num_neg:end), :);                        
            neg_confidences = neg_confidences(floor(1:num_passed/num_neg:end));
        else
            num_neg = num_passed;
            data = data(:, 1:num_pos+num_neg);
            confidences = confidences(1:num_pos+num_neg, 1);
        end
        data_locations(:, 3) = data_locations(:, 2) + med_length - 1;        
        for n = 1:num_neg
            data(:, num_pos+n) = neg_clips(data_locations(n, 1)).data(data_locations(n, 2):data_locations(n, 3));
            confidences(num_pos+n, 1) = neg_confidences(n);
        end
    end
    disp(['num pos: ' num2str(num_pos)  '      num neg: '  num2str(num_neg)]);    
    labels = [ones(num_pos, 1) ; -ones(num_neg, 1)];
        
        
            