% features(num_features, num_samples) is discretized
% partitions.{num_partitions}
% confidences(num_features).{value(num_partitions)}
function [classifier_confidences, threshold, total_confidences] = ...
                compute_classifier_confidences(features, labels, partitions)
    
    num_iter = 4;
    
    num_features = size(features, 1);

    num_samples = length(labels);
    num_pos = sum(labels == 1);
    num_neg = num_samples - num_pos;
    
    pos_weight = 1/(2*num_pos);
    neg_weight = 1/(2*num_neg);
    weights = [pos_weight*ones(num_pos, 1) ; neg_weight*ones(num_neg, 1)];
    
    for f = 1:num_features
        pos_hist(f).table = zeros(partitions(f).num_partitions, 1);
    end
    pos_hist(33).table = zeros(4, 1);
    pos_hist(34).table = zeros(16, 1);
    
    neg_hist = pos_hist;    
        
    total_confidences = zeros(num_samples, 1);
    prod_Z = 1;
    
    for i = 1:num_iter
    
        sum_neg_weights = sum(weights(1:num_pos));
        sum_pos_weights = sum(weights(num_pos+1:end));
        
        % compute weighted histogram
        for f = 1:num_features
            %disp(num2str(f))
            
            pos_hist(f).table = zeros(size(pos_hist(f).table));            
            for d = 1:num_pos
                pos_hist(f).table(features(f, d)) = pos_hist(f).table(features(f, d)) + weights(d);
            end
            pos_hist(f).table = (pos_hist(f).table+ 1/num_pos) / ...
                (sum_pos_weights + length(pos_hist(f).table)/num_pos);
            
            neg_hist(f).table = zeros(size(pos_hist(f).table));
            for d = num_pos+1:num_samples
                neg_hist(f).table(features(f, d)) = neg_hist(f).table(features(f, d)) + weights(d);
            end
            neg_hist(f).table = (neg_hist(f).table +1/num_neg) / ...
                (sum_neg_weights + length(neg_hist(f).table)/num_neg);
            
            tmp_classifier_confidences(f).values = log(pos_hist(f).table ./ neg_hist(f).table);
        end
        
        % compute data confidence
        data_confidences = zeros(num_samples, 1);
        for d = 1:num_samples
            for f = 1:num_features
                data_confidences(d) = data_confidences(d) + tmp_classifier_confidences(f).values(features(f, d));
            end
        end        
        
        % compute best alpha (minimize Z)
        alpha = fminbnd(@(x) compute_expected_confidence(x, labels, data_confidences, weights), 0.001, 1.0);
        
        % for debugging
        if 0 
            disp(['best alpha = ', num2str(alpha)]);
            Z = sum(exp(-alpha*labels.*data_confidences).*weights);
            prod_Z = prod_Z * Z;
            disp(['Z = ', num2str(Z) '  prod_Z = ', num2str(prod_Z)]);        
        end
        total_confidences = total_confidences + data_confidences*alpha;
        [training_error, threshold] = compute_min_error_rate(total_confidences, labels);
        disp(['training error = ', num2str(training_error)]);
        
        % compute new weights
        weights = reweight_data(labels, data_confidences, weights, alpha);

        for f = 1:num_features
            if i == 1
                classifier_confidences(f).values = tmp_classifier_confidences(f).values*alpha;
            else
                classifier_confidences(f).values = ...
                    classifier_confidences(f).values + tmp_classifier_confidences(f).values*alpha;
            end
        end
    end
            
        
        
        
        
        
        
        